from pytest import raises

from nibe.coil import Coil, CoilData
from nibe.connection import Connection
from nibe.exceptions import (
    ReadException,
    ReadExceptionGroup,
    ReadIOException,
    WriteIOException,
)


async def test_read_coils():
    coil1 = Coil(123, "test", "test", "u8")
    coil2 = Coil(231, "test2", "test", "u8")
    coil3 = Coil(231, "test3", "test", "u8")
    coil4 = Coil(233, "test3", "test", "u8")

    class MyConnection(Connection):
        async def read_coil(self, coil: Coil, timeout: float = ...) -> CoilData:
            if coil is coil2:
                raise ReadIOException(f"{coil.address}")
            if coil is coil4:
                raise ReadException(f"{coil.address}")
            return CoilData(coil, 1)

        async def verify_connectivity(self):
            return True

        async def write_coil(self, coil_data: CoilData, timeout: float = ...) -> None:
            raise WriteIOException()

    connection = MyConnection()

    coils = []
    with raises(ReadExceptionGroup) as excinfo:
        async for coil_data in connection.read_coils([coil1, coil2, coil3, coil4]):
            coils.append(coil_data.coil)
    assert str(excinfo.value) == "Failed to read some or all coils (231, 233)"
    assert coils == [coil1, coil3]
